@@ -10,7 +10,7 @@ use std::collections::HashMap;
 use bit_vec::BitVec;
 use serde::{Serialize, Deserialize};
 
-use crate::math::general::Increment;
+use crate::math::general::NumTools;
 /// Encoding trait for minimal code writing.
 pub trait Encode {
     /// Fully encodes a String.
@@ -29,6 +29,7 @@ pub trait Encode {
     /// 
     /// assert_eq!("Lorem Ipsum".to_owned(), dec);
     /// ```
+    #[must_use]
     fn full_encode(&self) -> (BitVec, Box<Node>);
 }
 /// Decoding trait for minimal code writing.
@@ -49,6 +50,7 @@ pub trait Decode {
     /// 
     /// assert_eq!("Lorem Ipsum".to_owned(), dec);
     /// ```
+    #[must_use]
     fn full_decode(&self) -> String;
 }
 
@@ -188,7 +190,7 @@ pub fn assign_codes(root: &Box<Node>,
 /// ```
 #[must_use]
 pub fn huffman_encode(s: &str, char_codes: &HashMap<char, BitVec>) -> BitVec {
-    let mut res: BitVec = BitVec::new();
+    let mut res: BitVec = BitVec::with_capacity(s.len());
     let mut t:   Option<&BitVec>;
 
     for c in s.chars() {
@@ -257,19 +259,21 @@ pub fn huffman_decode(bitvec: &BitVec, root: &Box<Node>) -> String {
 pub fn get_root(s: &str) -> Box<Node> {
     let frequency = get_frequency(s);
     let mut vec_nodes: Vec<Box<Node>> = frequency.iter().map(|x| Node::new_box(Node::new(*(x.1) as u128, Some(*(x.0))))).collect();
+    let mut a;
+    let mut b;
+    let mut c;
 
     while vec_nodes.len() > 1 {
         vec_nodes.sort_by(|a: &Box<Node>, b: &Box<Node>| (&(b.frequency)).cmp(&(a.frequency)));
-        let a:     Box<Node> = vec_nodes.pop().unwrap();
-        let b:     Box<Node> = vec_nodes.pop().unwrap();
-        let mut c: Box<Node> = Node::new_box(Node::new( a.frequency + b.frequency, None));
+        a = vec_nodes.pop().unwrap();
+        b = vec_nodes.pop().unwrap();
+        c = Node::new_box(Node::new( a.frequency + b.frequency, None));
 
         c.left  = Some(a);
         c.right = Some(b);
         vec_nodes.push(c);
     }
-    vec_nodes.pop()
-             .unwrap()
+    vec_nodes.pop().unwrap()
 }
 
 /// Writes the encoded message into 2 files.
