@@ -1,11 +1,15 @@
 //! Traits and functions for general purpose, everyday mathematics.
 //! Everything you need.
+
+use std::{convert::{TryInto, TryFrom}, ops::{Div, Sub, Add, Mul, AddAssign, SubAssign}};
+
 use crate::eval_postfix;
 
 use super::constants::{SQRT5, GOLDENRATIO};
 pub mod avg_impl;
 /// Trait for several kinds of averages.
 pub trait Averages<T> {
+    type Output;
     /// Calculate the arithmetic mean.
     /// # Returns
     /// A `f64`.
@@ -18,7 +22,7 @@ pub trait Averages<T> {
     /// assert_eq!(3.125, v.arithmetic_mean());
     /// ```
     #[must_use]
-    fn arithmetic_mean(&self) -> f64;
+    fn arithmetic_mean(&self) -> Self::Output;
     /// Calculate the harmonic mean.
     /// # Returns
     /// A `f64`.
@@ -31,7 +35,7 @@ pub trait Averages<T> {
     /// assert_eq!(2.318840579710145, v.harmonic_mean());
     /// ```
     #[must_use]
-    fn harmonic_mean(&self) -> f64;
+    fn harmonic_mean(&self) -> Self::Output;
     /// Calculate the median.
     /// # Returns
     /// A `f64`.
@@ -44,7 +48,7 @@ pub trait Averages<T> {
     /// assert_eq!(2.5, v.median());
     /// ```
     #[must_use]
-    fn median(&self) -> f64;
+    fn median(&self) -> Self::Output;
     /// Calculate the mode.
     /// # Returns
     /// A `T`.
@@ -58,7 +62,7 @@ pub trait Averages<T> {
     /// ```
     #[must_use]
     fn mode(&self) -> T;
-    /// Calculate the mode.
+    /// Calculate the mid range.
     /// # Returns
     /// A `T`.
     /// # Examples
@@ -70,7 +74,7 @@ pub trait Averages<T> {
     /// assert_eq!(3.5, v.mid_range());
     /// ```
     #[must_use]
-    fn mid_range(&self) -> f64;
+    fn mid_range(&self) -> Self::Output;
 }
 /// Trait for the Digits of a given number.
 pub trait NumDigits {
@@ -85,7 +89,7 @@ pub trait NumDigits {
     /// 
     /// assert_eq!(3u8, 12.cross_sum());
     /// assert_eq!(9u16, 342.cross_sum());
-    /// assert_eq!(52u64, 4928947234.cross_sum());
+    /// assert_eq!(52u64, 4928947234u64.cross_sum());
     /// ```
     #[must_use]
     fn cross_sum(&self) -> Self;
@@ -104,8 +108,33 @@ pub trait NumDigits {
     #[must_use]
     fn digits(&self) -> Vec<u8>;
 }
-/// Trait for mapping numbers to another number range.
-pub trait MapToNumRange<T> {
+/// Trait for several useful generic functions.
+pub trait NumTools<T> {
+    /// Determine whether `self` is in the interval `[start; end]`.
+    /// # Returns
+    /// A `bool`.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::math::general::NumTools;
+    /// 
+    /// assert_eq!(true, 5.is_in_range(0, 10));
+    /// assert_eq!(true, 5.is_in_range(0, 5));
+    /// assert_eq!(false, 3.14.is_in_range(5.0, 10.0));
+    /// ```
+    #[must_use]
+    fn is_in_range(&self, start: Self, end: Self) -> bool;
+    /// Determine whether `self` is in the interval `(start; end)`.
+    /// # Returns
+    /// A `bool`.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::math::general::NumTools;
+    /// 
+    /// assert_eq!(true, 5.is_in_range_exclusive(0, 10));
+    /// assert_eq!(false, 5.is_in_range_exclusive(0, 5));
+    /// ```
+    #[must_use]
+    fn is_in_range_exclusive(&self, start: Self, end: Self) -> bool;
     /// Maps a given number of a range onto another range.
     ///
     /// # Arguments
@@ -120,17 +149,65 @@ pub trait MapToNumRange<T> {
     ///
     /// # Examples
     /// ```
-    /// use lib_rapid::math::general::MapToNumRange;
+    /// use lib_rapid::math::general::NumTools;
     ///
     /// let result: f32 = 5.0.map_to(0., 10., 0., 1.); // Original value 5 in the range from 0-10
     /// assert_eq!(result, 0.5);
     /// ```
     #[must_use]
     fn map_to(&self, start1: T, end1: T, start2: T, end2: T) -> T;
-}
-
-/// Common powers.
-pub trait CommonPowers {
+    /// Increment a number by one.
+    /// # Returns
+    /// Nothing.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::math::general::NumTools;
+    /// let mut five: i32 = 5;
+    /// five.inc();
+    /// assert_eq!(five, 6);
+    /// ```
+    fn inc(&mut self);
+    /// Increment a number by a specified value.
+    /// # Arguments
+    /// * `n` - The value to be incremented by.
+    /// # Returns
+    /// Nothing.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::math::general::NumTools;
+    /// let mut five: i32 = 5;
+    /// five.inc_by(2);
+    /// assert_eq!(five, 7);
+    /// ```
+    fn inc_by(&mut self, n: Self);
+    /// Decrement a number by one.
+    /// # Returns
+    /// Nothing.
+    /// # Warning
+    /// Does not check for underflow.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::math::general::NumTools;
+    /// let mut five: i32 = 5;
+    /// five.dec();
+    /// assert_eq!(five, 4);
+    /// ```
+    fn dec(&mut self);
+    /// Decrement a number by a specified value.
+    /// # Arguments
+    /// * `n` - The value to be decremented by.
+    /// # Returns
+    /// Nothing.
+    /// # Warning
+    /// Does not check for underflow.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::math::general::NumTools;
+    /// let mut five: i32 = 5;
+    /// five.dec_by(2);
+    /// assert_eq!(five, 3);
+    /// ```
+    fn dec_by(&mut self, n: Self);
     /// Square a number.
     /// # Returns
     /// The square of the number.
@@ -138,7 +215,7 @@ pub trait CommonPowers {
     /// This function does not check if overflow occurs.
     /// # Examples
     /// ```
-    ///use lib_rapid::math::general::CommonPowers;
+    ///use lib_rapid::math::general::NumTools;
     ///let i = 12;
     ///let res = i.square();
     ///
@@ -154,7 +231,7 @@ pub trait CommonPowers {
     /// This function does not check if overflow occurs.
     /// # Examples
     /// ```
-    ///use lib_rapid::math::general::CommonPowers;
+    ///use lib_rapid::math::general::NumTools;
     ///let i = 12;
     ///let res = i.cube();
     ///
@@ -163,210 +240,97 @@ pub trait CommonPowers {
     #[must_use]
     fn cube(&self) -> Self;
 }
-/// Trait for incrementing by value. Shorthand syntax for `x += y;`.
-pub trait Increment {
-    /// Increment a number by one.
-    /// # Returns
-    /// Nothing.
-    /// # Examples
-    /// ```
-    /// use lib_rapid::math::general::Increment;
-    /// let mut five: i32 = 5;
-    /// five.inc();
-    /// assert_eq!(five, 6);
-    /// ```
-    fn inc(&mut self);
-    /// Increment a number by a specified value.
-    /// # Arguments
-    /// * `n` - The value to be incremented by.
-    /// # Returns
-    /// Nothing.
-    /// # Examples
-    /// ```
-    /// use lib_rapid::math::general::Increment;
-    /// let mut five: i32 = 5;
-    /// five.inc_by(2);
-    /// assert_eq!(five, 7);
-    /// ```
-    fn inc_by(&mut self, n: Self);
-}
-/// Trait for incrementing by value. Shorthand syntax for `x -= y;`.
-pub trait Decrement {
-    /// Decrement a number by one.
-    /// # Returns
-    /// Nothing.
-    /// # Warning
-    /// Does not check for underflow.
-    /// # Examples
-    /// ```
-    /// use lib_rapid::math::general::Decrement;
-    /// let mut five: i32 = 5;
-    /// five.dec();
-    /// assert_eq!(five, 4);
-    /// ```
-    fn dec(&mut self);
-    /// Decrement a number by a specified value.
-    /// # Arguments
-    /// * `n` - The value to be decremented by.
-    /// # Returns
-    /// Nothing.
-    /// # Warning
-    /// Does not check for underflow.
-    /// # Examples
-    /// ```
-    /// use lib_rapid::math::general::Decrement;
-    /// let mut five: i32 = 5;
-    /// five.dec_by(2);
-    /// assert_eq!(five, 3);
-    /// ```
-    fn dec_by(&mut self, n: Self);
-}
 
-impl<T: std::ops::AddAssign + From<u8>> Increment for T {
-    fn inc(&mut self) {
-        *self += 1u8.into();
+impl<T: std::cmp::PartialOrd +
+        Sub<Output = T> +
+        Add<Output = T> +
+        Div<Output = T> +
+        Mul<Output = T> +
+        From<u8> +
+        Copy +
+        SubAssign +
+        AddAssign> NumTools<T> for T {
+    #[inline]
+    fn is_in_range(&self, start: Self, end: Self) -> bool {
+        self >= &start && self <= &end
     }
 
-    fn inc_by(&mut self, n: Self) {
-        *self += n;
+    #[inline]
+    fn is_in_range_exclusive(&self, start: Self, end: Self) -> bool {
+        self > &start && self < &end
     }
-}
 
-impl<T: std::ops::SubAssign + From<u8>> Decrement for T {
+    #[inline]
+    fn map_to(&self, start1: T, end1: T, start2: T, end2: T) -> T {
+        eval_postfix!(start2 end2 start2 - + (*self) start1 - end1 / start1 - *)
+    }
+
+    #[inline]
     fn dec(&mut self) {
         *self -= 1u8.into();
     }
 
+    #[inline]
     fn dec_by(&mut self, n: Self) {
         *self -= n;
     }
-}
 
-impl NumDigits for u8 {
-    fn cross_sum(&self) -> Self {
-        self.digits().iter().sum()
+    #[inline]
+    fn inc(&mut self) {
+        *self += 1u8.into();
     }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u8{ return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(3);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n);
-        }
-        digits
+    #[inline]
+    fn inc_by(&mut self, n: Self) {
+        *self += n;
     }
-}
-
-impl NumDigits for u16 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
-        }
-        res
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u16 { return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(5);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
-        }
-        digits
-    }
-}
-
-impl NumDigits for u32 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
-        }
-        res
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u32 { return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(10);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
-        }
-        digits
-    }
-}
-
-impl NumDigits for u64 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
-        }
-        res
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u64 { return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(20);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
-        }
-        digits
-    }
-}
-
-impl NumDigits for u128 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
-        }
-        res
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u128 { return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(39);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
-        }
-        digits
-    }
-}
-
-impl<T: std::ops::Add<Output = T> + 
-        std::ops::Sub<Output = T> + 
-        std::ops::Mul<Output = T> + 
-        std::ops::Div<Output = T> + 
-        Copy> MapToNumRange<T> for T {
-            fn map_to(&self, start1: T, end1: T, start2: T, end2: T) -> T {
-                eval_postfix!(start2 end2 start2 - + (*self) start1 - end1 / start1 - *)
-            }
-        }
-
-impl<T: std::ops::Mul<Output = T> + Copy> CommonPowers for T {
+    #[inline]
     fn square(&self) -> Self {
         *self * *self
     }
-
+    #[inline]
     fn cube(&self) -> Self {
         *self * *self * *self
     }
 }
+
+impl<'a, T: std::cmp::PartialEq +
+        std::ops::DivAssign + std::cmp::PartialOrd +
+        std::ops::Rem<Output = T> + Copy +
+        std::ops::AddAssign +
+        Div<Output = T> +
+        Sub<Output = T> +
+        Add<Output = T> +
+        Mul<Output = T> +
+        AddAssign +
+        SubAssign>
+    NumDigits
+    for T
+    where
+        u8: TryFrom<T>,
+        T: From<u8> {
+    fn cross_sum(&self) -> Self {
+        let d: Vec<u8> = self.digits();
+        let mut res: Self = 0.into();
+        for i in d {
+            res.inc_by(i.into());
+        }
+        res
+    }
+
+    fn digits(&self) -> Vec<u8> {
+        if self == &0u8.into() { return vec![0]; }
+        let mut v: Self = *self;
+        let mut digits: Vec<u8> = Vec::with_capacity(39);
+        while v > 0.into() {
+            let n: u8 = unsafe { (v % 10.into()).try_into().unwrap_unchecked() };
+            if n == 255
+            { panic!("Oops! Something went wrong. please contact the developers using the error code 0x00001.")}
+            v /= 10.into();
+            digits.push(n);
+        }
+        digits
+    }
+}
 /// Compute the nth-root of a number.
 /// # Arguments
 /// * `degree` - the `f64` specifying the `n`.
@@ -377,36 +341,23 @@ impl<T: std::ops::Mul<Output = T> + Copy> CommonPowers for T {
 /// ```
 /// use lib_rapid::math::general::nth_root;
 /// use lib_rapid::math::constants;
-/// assert_eq!(1.4422495703074085, nth_root(3.0, 3.0));
+/// assert_eq!(constants::CUBEROOT3, nth_root(3.0, 3.0));
 /// ```
 /// ```
 /// use lib_rapid::math::general::nth_root;
 /// use lib_rapid::math::constants;
-/// assert_eq!(1.414213562373095, nth_root(2.0, 2.0));
+/// assert_eq!(constants::SQRT2, nth_root(2.0, 2.0));
 /// ```
+#[must_use]
+#[inline]
 pub fn nth_root(degree: f64, radicand: f64) -> f64 {
-    // Using Newtons method
-    let     p:             f64 = 1e-15_f64;
-    let mut initial_guess: f64 = radicand / degree;
-    let mut next_guess:    f64;
-  
-    loop {
-       next_guess = ((degree - 1.0) * initial_guess 
-                    + radicand / 
-                    f64::powf(initial_guess, degree - 1.0))
-                    / degree; // Guess the next guess
-
-       if (next_guess - initial_guess).abs() <= (initial_guess * p).abs()
-       { return next_guess; }
-
-       initial_guess = next_guess;
-    }
+    radicand.powf(degree.recip())
 }
 /// Computes th nth fibonacci number (up to 186th) accurately using the fastest available computing method.
 /// # Arguments
 /// * `n` - the nth-fibonacci number to be computed.
 /// # How does it work?
-/// This function uses the function `(½(1 + √5))ⁿ / √5` for all numbers `n ≤ 74`, as this is the maximum precision for that formula. For every number above that, it uses the iterative approach.
+/// This function uses the function `φⁿ / √5` for all numbers `n < 76`, as this is the maximum precision for that formula. For every number above that, it uses the iterative approach.
 /// # Examples
 /// ```
 /// use lib_rapid::math::general::nth_fibonacci;
@@ -414,17 +365,18 @@ pub fn nth_root(degree: f64, radicand: f64) -> f64 {
 /// assert_eq!(1304969544928657, nth_fibonacci(74));
 /// assert_eq!(332825110087067562321196029789634457848, nth_fibonacci(186));
 /// ```
+#[must_use]
 pub fn nth_fibonacci(n: u128) -> u128 {
-    if n >= 187 
+    if n > 186
     { panic!("Error: The 187th Fibonacci number and all above are not allowed, as they would cause a overflow in the u128 type."); }
-    if n <= 75 {
+    if n < 76 {
         return ( GOLDENRATIO.powi(n as i32) / SQRT5 ).round() as u128;
     }
 
     let mut x: u128;
-    let mut y: u128 = 0;
-    let mut z: u128 = 1;
-    let mut i: u128 = 1;
+    let mut y: u128 = 1304969544928657;
+    let mut z: u128 = 2111485077978050;
+    let mut i: u128 = 75;
 
     while i < n {
         x = y;
@@ -433,5 +385,5 @@ pub fn nth_fibonacci(n: u128) -> u128 {
         i.inc();
     }
 
-    return z;
+    z
 }
\ No newline at end of file
